/****************************************************************************
** Meta object code from reading C++ file 'kcardscene.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kpat-26.04.2/src/libkcardgame/kcardscene.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kcardscene.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10KCardSceneE_t {};
} // unnamed namespace

template <> constexpr inline auto KCardScene::qt_create_metaobjectdata<qt_meta_tag_ZN10KCardSceneE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KCardScene",
        "cardClicked",
        "",
        "KCard*",
        "card",
        "cardDoubleClicked",
        "cardRightClicked",
        "pileClicked",
        "KCardPile*",
        "pile",
        "pileDoubleClicked",
        "pileRightClicked",
        "cardAnimationDone",
        "keyboardFocusLeft",
        "keyboardFocusRight",
        "keyboardFocusUp",
        "keyboardFocusDown",
        "keyboardFocusCancel",
        "keyboardFocusSelect"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'cardClicked'
        QtMocHelpers::SignalData<void(KCard *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'cardDoubleClicked'
        QtMocHelpers::SignalData<void(KCard *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'cardRightClicked'
        QtMocHelpers::SignalData<void(KCard *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'pileClicked'
        QtMocHelpers::SignalData<void(KCardPile *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'pileDoubleClicked'
        QtMocHelpers::SignalData<void(KCardPile *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'pileRightClicked'
        QtMocHelpers::SignalData<void(KCardPile *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'cardAnimationDone'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'keyboardFocusLeft'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'keyboardFocusRight'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'keyboardFocusUp'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'keyboardFocusDown'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'keyboardFocusCancel'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'keyboardFocusSelect'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KCardScene, qt_meta_tag_ZN10KCardSceneE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KCardScene::staticMetaObject = { {
    QMetaObject::SuperData::link<QGraphicsScene::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KCardSceneE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KCardSceneE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10KCardSceneE_t>.metaTypes,
    nullptr
} };

void KCardScene::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KCardScene *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->cardClicked((*reinterpret_cast<std::add_pointer_t<KCard*>>(_a[1]))); break;
        case 1: _t->cardDoubleClicked((*reinterpret_cast<std::add_pointer_t<KCard*>>(_a[1]))); break;
        case 2: _t->cardRightClicked((*reinterpret_cast<std::add_pointer_t<KCard*>>(_a[1]))); break;
        case 3: _t->pileClicked((*reinterpret_cast<std::add_pointer_t<KCardPile*>>(_a[1]))); break;
        case 4: _t->pileDoubleClicked((*reinterpret_cast<std::add_pointer_t<KCardPile*>>(_a[1]))); break;
        case 5: _t->pileRightClicked((*reinterpret_cast<std::add_pointer_t<KCardPile*>>(_a[1]))); break;
        case 6: _t->cardAnimationDone(); break;
        case 7: _t->keyboardFocusLeft(); break;
        case 8: _t->keyboardFocusRight(); break;
        case 9: _t->keyboardFocusUp(); break;
        case 10: _t->keyboardFocusDown(); break;
        case 11: _t->keyboardFocusCancel(); break;
        case 12: _t->keyboardFocusSelect(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KCardScene::*)(KCard * )>(_a, &KCardScene::cardClicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCardScene::*)(KCard * )>(_a, &KCardScene::cardDoubleClicked, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCardScene::*)(KCard * )>(_a, &KCardScene::cardRightClicked, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCardScene::*)(KCardPile * )>(_a, &KCardScene::pileClicked, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCardScene::*)(KCardPile * )>(_a, &KCardScene::pileDoubleClicked, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCardScene::*)(KCardPile * )>(_a, &KCardScene::pileRightClicked, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCardScene::*)()>(_a, &KCardScene::cardAnimationDone, 6))
            return;
    }
}

const QMetaObject *KCardScene::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KCardScene::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10KCardSceneE_t>.strings))
        return static_cast<void*>(this);
    return QGraphicsScene::qt_metacast(_clname);
}

int KCardScene::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QGraphicsScene::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void KCardScene::cardClicked(KCard * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KCardScene::cardDoubleClicked(KCard * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KCardScene::cardRightClicked(KCard * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KCardScene::pileClicked(KCardPile * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KCardScene::pileDoubleClicked(KCardPile * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void KCardScene::pileRightClicked(KCardPile * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void KCardScene::cardAnimationDone()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
