/****************************************************************************
** Meta object code from reading C++ file 'dealer.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kpat-26.04.2/src/dealer.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dealer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11DealerSceneE_t {};
} // unnamed namespace

template <> constexpr inline auto DealerScene::qt_create_metaobjectdata<qt_meta_tag_ZN11DealerSceneE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DealerScene",
        "undoPossible",
        "",
        "poss",
        "redoPossible",
        "newCardsPossible",
        "gameInProgress",
        "inProgress",
        "demoActive",
        "active",
        "hintActive",
        "dropActive",
        "updateMoves",
        "moves",
        "solverStateChanged",
        "text",
        "newDeal",
        "cardsPickedUp",
        "cardsPutDown",
        "startNew",
        "dealNumber",
        "undo",
        "redo",
        "stop",
        "drawDealRowOrRedeal",
        "tryAutomaticMove",
        "KCard*",
        "card",
        "takeState",
        "animationDone",
        "newCards",
        "drop",
        "stopAndRestartSolver",
        "slotSolverEnded",
        "slotSolverFinished",
        "result",
        "demo",
        "showWonMessage"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'undoPossible'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'redoPossible'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'newCardsPossible'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'gameInProgress'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'demoActive'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'hintActive'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'dropActive'
        QtMocHelpers::SignalData<void(bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'updateMoves'
        QtMocHelpers::SignalData<void(int)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 13 },
        }}),
        // Signal 'solverStateChanged'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Signal 'newDeal'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cardsPickedUp'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cardsPutDown'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startNew'
        QtMocHelpers::SlotData<void(int)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 20 },
        }}),
        // Slot 'startNew'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'undo'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'redo'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'stop'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'drawDealRowOrRedeal'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'tryAutomaticMove'
        QtMocHelpers::SlotData<bool(KCard *)>(25, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Slot 'takeState'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'animationDone'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'newCards'
        QtMocHelpers::SlotData<bool()>(30, 2, QMC::AccessProtected, QMetaType::Bool),
        // Slot 'drop'
        QtMocHelpers::SlotData<bool()>(31, 2, QMC::AccessProtected, QMetaType::Bool),
        // Slot 'stopAndRestartSolver'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSolverEnded'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSolverFinished'
        QtMocHelpers::SlotData<void(int)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 35 },
        }}),
        // Slot 'demo'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showWonMessage'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DealerScene, qt_meta_tag_ZN11DealerSceneE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DealerScene::staticMetaObject = { {
    QMetaObject::SuperData::link<KCardScene::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11DealerSceneE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11DealerSceneE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11DealerSceneE_t>.metaTypes,
    nullptr
} };

void DealerScene::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DealerScene *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->undoPossible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->redoPossible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->newCardsPossible((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->gameInProgress((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->demoActive((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->hintActive((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->dropActive((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->updateMoves((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->solverStateChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->newDeal(); break;
        case 10: _t->cardsPickedUp(); break;
        case 11: _t->cardsPutDown(); break;
        case 12: _t->startNew((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->startNew(); break;
        case 14: _t->undo(); break;
        case 15: _t->redo(); break;
        case 16: _t->stop(); break;
        case 17: _t->drawDealRowOrRedeal(); break;
        case 18: { bool _r = _t->tryAutomaticMove((*reinterpret_cast<std::add_pointer_t<KCard*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: _t->takeState(); break;
        case 20: _t->animationDone(); break;
        case 21: { bool _r = _t->newCards();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->drop();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 23: _t->stopAndRestartSolver(); break;
        case 24: _t->slotSolverEnded(); break;
        case 25: _t->slotSolverFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 26: _t->demo(); break;
        case 27: _t->showWonMessage(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KCard* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DealerScene::*)(bool )>(_a, &DealerScene::undoPossible, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DealerScene::*)(bool )>(_a, &DealerScene::redoPossible, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DealerScene::*)(bool )>(_a, &DealerScene::newCardsPossible, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DealerScene::*)(bool )>(_a, &DealerScene::gameInProgress, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DealerScene::*)(bool )>(_a, &DealerScene::demoActive, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DealerScene::*)(bool )>(_a, &DealerScene::hintActive, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DealerScene::*)(bool )>(_a, &DealerScene::dropActive, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DealerScene::*)(int )>(_a, &DealerScene::updateMoves, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (DealerScene::*)(const QString & )>(_a, &DealerScene::solverStateChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (DealerScene::*)()>(_a, &DealerScene::newDeal, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (DealerScene::*)()>(_a, &DealerScene::cardsPickedUp, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (DealerScene::*)()>(_a, &DealerScene::cardsPutDown, 11))
            return;
    }
}

const QMetaObject *DealerScene::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DealerScene::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11DealerSceneE_t>.strings))
        return static_cast<void*>(this);
    return KCardScene::qt_metacast(_clname);
}

int DealerScene::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KCardScene::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    }
    return _id;
}

// SIGNAL 0
void DealerScene::undoPossible(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void DealerScene::redoPossible(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void DealerScene::newCardsPossible(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void DealerScene::gameInProgress(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void DealerScene::demoActive(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void DealerScene::hintActive(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void DealerScene::dropActive(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void DealerScene::updateMoves(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void DealerScene::solverStateChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void DealerScene::newDeal()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void DealerScene::cardsPickedUp()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void DealerScene::cardsPutDown()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}
QT_WARNING_POP
